#!/usr/bin/python
# coding=utf-8

from __future__ import division # division of two integers like 20/100 has a float result instead of 0

"""
    Utils for working with colors.
    Source: stackoverflow.com/questions/214359/converting-hex-color-to-rgb-and-vice-versa
"""

import string
import textwrap
    
def isHEX(color):
    """
        Checks if the value is a valid hex color.
        
        >>> isHex('#FFFFFF')
        True
        >>> isHex('#FFFFFG')
        False
    """
    
    # Cuts leading #
    if color[0] == '#':
        color = color[1:]
    
    # Checks the length
    if len(color) not in [3, 6]:
        return False
    
    # Checks the characters
    if not all(c in string.hexdigits for c in color.lower()):
        return False
    
    return True





def isRGB(color):
    """
        Checks if the value is a valid RGB color.
        
        >>> isRGB((255, 255, 255))
        True
        >>> isRGB((0, 0, 0))
        True
        >>> isRGB((255, 255, 256))
        False
    """
    
    # Checks the red, green and blue values of the color
    for value in color:
        if not 0 <= value <= 255:
            return False
    
    return True





def isRGBA(color):
    """
        Checks if the value is a valid RGBA color.
        
        >>> isRGBA((1.0, 1.0, 1.0, 1.0))
        True
        >>> isRGBA((0.0, 0.0, 0.0, 0.5))
        True
        >>> isRGBA((255, 255, 255))
        False
    """
    
    # Checks the length
    if len(color) != 4:
        return False
    
    # Checks the red, green and blue values of the color
    for value in color[0:2]:
        if not 0 <= value <= 255:
            return False
    
    # Checks that the alpha value is in a valid range
    if not 0 <= color[3] <= 1:
        return False
    
    return True



    

def rgb_to_hex(value):
    """
        Convert RGB value (as three numbers each ranges from 0 to 255) to hex format.
        
        >>> rgb_to_hex((255, 255, 255))
        '#FFFFFF'
        >>> rgb_to_hex((0, 0, 0))
        '#000000'
    """
    
    # Checks if the value is a valid RGB color
    if not isRGB(value):
        raise ValueError('Each color value must be between 0 and 255')
    
    return '#{0:02X}{1:02X}{2:02X}'.format(value[0], value[1], value[2])





def hex_to_rgb(value):
    """
        Convert color's value to RGB format.
        
        >>> hex_to_rgb('#FFFFFF')
        (255, 255, 255)
        >>> hex_to_rgb('#000')
        (0, 0, 0)
    
    """
    
    # Checks if the value is a valid hex color
    if not isHEX(value):
        raise ValueError('{} is not a valid hex color'.format(value))

    if value[0] == '#':
        value = value[1:]
    
    if len(value) == 3:
        value = ''.join(i * 2 for i in value)
    
    return tuple(int(i, 16) for i in textwrap.wrap(value, 2))





def rgb_to_rgba(value, alpha=1.0):
    """
        Convert RGB value (as three numbers each ranges from 0 to 255) to RGBA format.
        
        >>> rgb_to_rgba((255, 255, 255))
        (1.0, 1.0, 1.0, 1.0)
        >>> rgb_to_rgba((255, 175, 0), 0.5)
        (1.0, 0.6862745098039216, 0.0, 0.5)

    """
    
    # Checks if the value is a valid RGB color
    if not isRGB(value):
        raise ValueError('Each color value must be between 0 and 255')
    
    return (value[0]/255, value[1]/255, value[2]/255, alpha)
    
    




def hex_to_rgba(value, alpha=1.0):
    """
        Convert color's value to RGBA format.
        
        >>> hex_to_rgba('#FFFFFF')
        (1.0, 1.0, 1.0, 1.0)
        >>> hex_to_rgba('#000', 0.5)
        (0.0, 0.0, 0.0, 0.5)

    """
    
    # Checks if the value is a valid hex color
    if not isHEX(value):
        raise ValueError('{} is not a valid hex color'.format(value))
        
    rgb = hex_to_rgb(value)
    
    return rgb_to_rgba(rgb, alpha)
    




def rgba_to_rgb(value):
    """
        Convert RGBA value (as three numbers each ranges from 0 to 255) to RGB format.
        
        >>> rgba_to_rgb((1.0, 1.0, 1.0, 1.0))
        (255, 255, 255)
        >>> rgba_to_rgb((0.0, 0.0, 0.0, 0.5))
        (0, 0, 0)

    """
    
    # Checks if the value is a valid RGBA color
    if not isRGBA(value):
        raise ValueError('Each color value must be between 0 and 255 and alpha between 0 and 1')
    
    return (int(value[0]*255), int(value[1]*255), int(value[2]*255))
    




def rgba_to_hex(value):
    """
        Convert RGBA value (as three numbers each ranges from 0 to 255) to RGB format.
        
        >>> rgba_to_hex((1.0, 1.0, 1.0, 1.0))
        '#FFFFFF'
        >>> rgba_to_hex((0.0, 0.0, 0.0, 0.5))
        '#000000'

    """
    
    # Checks if the value is a valid RGBA color
    if not isRGBA(value):
        raise ValueError('Each color value must be between 0 and 255 and alpha between 0 and 1')
    
    return rgb_to_hex( rgba_to_rgb(value) )





if __name__ == '__main__':
    import doctest
    doctest.testmod()